#ifndef H_INT_COEFFICIENT_FUNCTION
# define H_INT_COEFFICIENT_FUNCTION
#include "comp.hpp"
#include "ArgumentMarker.hpp"
#include <iostream>

using namespace ngcomp;

namespace ngfem 
{
	extern LocalHeap glh;

	class IntCoefficientFunction : public CoefficientFunction 
	{
		std::shared_ptr<ngcomp::MeshAccess> mesh;
		std::shared_ptr<CoefficientFunction> cf;

	public:
		IntCoefficientFunction(
			std::shared_ptr<ngcomp::MeshAccess> mesh,
			std::shared_ptr<CoefficientFunction> cf
		) : CoefficientFunction(1)
		{
			this->mesh = mesh; 
			this->cf = cf;
		}

		virtual double Evaluate(const BaseMappedIntegrationPoint& mip) const override
		{
			// set the marker for the integration points
			FlatVector<> point = mip.GetPoint();
			double x = point[0];
			double y = point[1];
			this->cf->TraverseTree( 
				[&] (CoefficientFunction& node) 
				{
					if(dynamic_cast<XArgumentMarker*>(&node))  
						dynamic_cast<ParameterCoefficientFunction*>(&node)->SetValue(x);
					if(dynamic_cast<YArgumentMarker*>(&node)) 
						dynamic_cast<ParameterCoefficientFunction*>(&node)->SetValue(y);
				}
			);
			// do integration over whole domain
			int dim = this->cf->Dimension(); // in case of vector-values cf
			Vector<> sum(dim);
			sum = 0.0; // init to zero component-wise
			this->mesh->IterateElements(
				VOL, 
				glh,
				[&] (Ngs_Element el, LocalHeap& lh)
				{
					auto& trafo = this->mesh->GetTrafo(el, lh); // transformation into
															    // physical coords
					FlatVector<> hsum(dim, lh); // holds int approx on el
											   // for every component of cf (1 in our case)
					hsum = 0.0; 
					IntegrationRule ir(trafo.GetElementType(), 2); // quad points
					BaseMappedIntegrationRule& mir = trafo(ir, lh); // transf quad points
					FlatMatrix<> values(ir.Size(), dim, lh);
					this->cf->Evaluate(mir, values); // apply on all transf quad points
					// over all integration points, every component of a row
					// is a component of the coefficient function
					for (int i=0; i<values.Height(); i++)
						hsum += mir[i].GetWeight() * values.Row(i);
					// sum up 
					for (size_t i=0; i<dim; i++)
						AtomicAdd(sum(i), hsum(i));
				}
			);
            sum(0) = this->mesh->GetCommunicator().AllReduce(sum(0), MPI_SUM);
			double res = sum(0);
			std::cout << "return " << res << std::endl;
			return res;
		}
	};

	void ExportIntCoefficientFunction(py::module m);
}

#endif
